function results=nwest3_en(y,x,T)
% PURPOSE: computes Newey-West adjusted heteroscedastic-serial
%          consistent Least-squares Regression
% has table with date and forecast object date
% reports t-stat with private forecast = 0 &
%  Fed forecast = 1
% calculates forecast errors
%---------------------------------------------------
% USAGE: results = nwest(y,x,nlag)
% where: y = dependent variable vector (nobs x 1)
%        x = independent variables matrix (nobs x nvar)
%        T = table of data
%     nlag = lag length to use
%---------------------------------------------------
% RETURNS: a structure
%        results.meth  = 'newlyw'
%        results.beta  = bhat
%        results.tstat = t-stats
%        results.yhat  = yhat
%        results.resid = residuals
%        results.sige  = e'*e/(n-k)
%        results.rsqr  = rsquared
%        results.rbar  = rbar-squared
%        results.dw    = Durbin-Watson Statistic
%        results.nobs  = nobs
%        results.nvar  = nvars
%        results.y     = y data vector
%        results.x     = x data vector (nobs x 1)
%        results.nwerr = NW standard error
%        results.covar = variance covariance matrix
% --------------------------------------------------
% SEE ALSO: nwest_d, prt(results), plt(results)
%---------------------------------------------------
% References:  Gallant, R. (1987),
%  "Nonlinear Statistical Models," pp.137-139.
%---------------------------------------------------

% written by:
% James P. LeSage, Dept of Economics
%
% Edited from Spatial Econometrics Toolbox by James P. LeSage

T_new = rmmissing(T);


% Remove missing values, if any
wasnan = (isnan(y) | any(isnan(x),2));
havenans = any(wasnan);
if havenans
   y(wasnan) = [];
   x(wasnan,:) = [];
   n = length(y);
end

if (nargin ~= 3); error('Wrong # of arguments to nwest'); end;
%results.date = x(:,1);
%x = x(:,2:4);
[nobs nvar] = size(x);

results.meth    = 'nwest';
results.y       = y;
results.x       = x;
results.nobs    = nobs;
results.nvar    = nvar;
FE              = y - x(:,2:3); %forecast errors
results.FE      = FE;
se              = FE.^2; %squared forecast error
results.MFE     = mean(FE); %mean forecast error
results.MSFE    = mean(se); %mean squared FE
results.RMSFE   = sqrt(results.MSFE); %root mean squared FE
results.medianSFE = median(se); %median squared FE
results.iqr     = iqr(FE); %interquartile range of error; difference between the 75th and 25th percentile of the data
    

xpxi = inv(x'*x);
results.beta    = xpxi*(x'*y);
results.yhat    = x*results.beta;
results.resid   = y - results.yhat;
sigu = results.resid'*results.resid;
results.sige    = sigu/(nobs-nvar);

p =  x*xpxi*x'; %Projection matrix
results.lev   = diag(p); %leverage


se_e = sqrt(results.sige*(1-results.lev));
results.se_e = se_e;
stdres = results.resid ./ se_e;
results.stdres = stdres;
stdres2 = stdres .* stdres;
h1h = results.lev ./ (1-results.lev);
d = (1/(nvar+1))*stdres2 .* h1h;
results.d = d; %Cook's distance

T_new = addvars(T_new,results.lev,results.d);
T_new.Properties.VariableNames{'Var5'} = 'leverage';
T_new.Properties.VariableNames{'Var6'} = 'Cookd';

results.T       = T_new;

A = size(d,1);
avg_lev = (1/nobs)*sum(results.lev);
results.avg_lev = avg_lev;
a = 2*avg_lev;
high_lev = zeros(A,1);
for i = 1:nobs
    if results.lev(i,1) > a
        high_lev(i,1) = 1;
    end
end
high_lev_sum = sum(high_lev); % total high leverage

d_mod = zeros(A,1);
for i = 1:nobs
    if d(i,1) > 0.5
    d_mod(i,1) = 1;
    end
end    
d_mod_sum = sum(d_mod); %total flagged moderate


d_extreme = zeros(A,1);
for i = 1:nobs
    if d(i,1) > 1
        d_extreme(i,1) = 1;
    end
end   
d_extreme_sum = sum(d_extreme); %total flagged extreme

results.high_lev =high_lev;
results.d_mod = d_mod;
results.d_extreme = d_extreme;
results.high_lev_sum = high_lev_sum;
results.d_mod_sum = d_mod_sum;
results.d_extreme_sum = d_extreme_sum;

%results.xiei = x'*results.resid;
%results.xxxe = xpxi*results.xiei;
%results.dfbeta = xpxi*results.xiei/(1-results.diagp);

%%% NW 1994 suggests a bandwidth of {4(N/100)^(2/9)} + 1
nwaa = (nobs/100)^(2/9);
nwnlagaa = (4*nwaa) + 1;
nwnlag = round(nwnlagaa);

%%%%%% Lazarus, Lewis, Stock, and Watson argue for (1.3*N^(1/2))+1
%nwaa = (nobs)^(1/2);
%nwnlagaa = (1.3*nwaa) + 1;
%nwnlag = round(nwnlagaa);

% perform Newey-West correction
emat = [];
for i=1:nvar;
emat = [emat
        results.resid'];
end;
       
    hhat=emat.*x';
    G=zeros(nvar,nvar); w=zeros(2*nwnlag+1,1);
    a=0;

    while a~=nwnlag+1;
        ga=zeros(nvar,nvar);
        w(nwnlag+1+a,1)=(nwnlag+1-a)/(nwnlag+1);
        za=hhat(:,(a+1):nobs)*hhat(:,1:nobs-a)';
          if a==0;
           ga=ga+za;
          else
           ga=ga+za+za';
          end;
        G=G+w(nwnlag+1+a,1)*ga;
        a=a+1;
    end; % end of while
    
        V=xpxi*G*xpxi;
        nwerr= sqrt(diag(V));

results.tstat(1:2,1) = results.beta(1:2,1)./nwerr(1:2,1); % Newey-West t-statistics
results.tstat(3,1) = (results.beta(3,1) - 1)./nwerr(3,1); % Newey-West t-statistics Fed = 1
results.pvalue = tdis_prb(results.tstat,nobs-nvar); % find t-stat probabilities
results.nwerr = nwerr; % NW standard error
results.covar = V; %variance covariance matrix
ym = y - ones(nobs,1)*mean(y);
rsqr1 = sigu;
rsqr2 = ym'*ym;
results.rsqr = 1.0 - rsqr1/rsqr2; % r-squared
rsqr1 = rsqr1/(nobs-nvar);
rsqr2 = rsqr2/(nobs-1.0);
results.rbar = 1 - (rsqr1/rsqr2); % rbar-squared
ediff = results.resid(2:nobs) - results.resid(1:nobs-1);
results.dw = diag((ediff'*ediff)./(sigu))'; % durbin-watson

