function results=rmsfe(y,x)
% PURPOSE: computes Root mean square forecast errors for multiple forecasts
%           for the same sample and horizon 
%---------------------------------------------------
% USAGE: results = rmsfe(y,x)
% where: y = actual/realized variable vector (nobs x 1)
%        x = forecast variables matrix (nobs x nvar)
%---------------------------------------------------
% RETURNS: a structure
%        results.meth  = 'rmsfe'
%        results.nobs  = nobs
%        results.nvar  = nvars
%        results.y     = y data vector
%        results.x     = x data vector
%        results.MFE   =  mean forecast error
%        results.MAFE  =  mean absolute FE
%        results.MSFE  = mean squared FE
%        results.RMSFE = root mean squared FE
% --------------------------------------------------
% NOTES: uses function mcov() included in this file
% --------------------------------------------------
% SEE ALSO: hwhite_d, prt(results), plt(results)
%---------------------------------------------------
% References: H. White 1980, Econometrica Vol. 48 pp. 818-838. 
%---------------------------------------------------

% written by:
% James P. LeSage, Dept of Economics
% University of Toledo
% 2801 W. Bancroft St,
% Toledo, OH 43606
% jlesage@spatial-econometrics.com

% Edited from Spatial Econometrics Toolbox by James P. LeSage

% Remove missing values, if any
wasnan = (isnan(y) | any(isnan(x),2));
havenans = any(wasnan);
if havenans
   y(wasnan) = [];
   x(wasnan,:) = [];
   n = length(y);
end

if (nargin ~= 2); error('Wrong # of arguments to rmsfe'); end;

[nobs nvar] = size(x);

results.meth    = 'rmsfe';
results.y       = y;
results.x       = x;
results.nobs    = nobs;
results.nvar    = nvar;
%{
    xp = x(:,1);
    e = y - xp; %forecast error
    ae = abs(e); %absolute forecast error
    se = e.^2; %squared forecast error

    results.MFE = mean(e); %mean forecast error
    results.MAFE = mean(ae); %mean absolute FE
    results.MSFE = mean(se); %mean squared FE
    results.RMSFE = sqrt(results.MSFE); %root mean squared FE
%}
for i=1:nvar
    xp = x(:,i);
    e = y - xp; %forecast error
    ae = abs(e); %absolute forecast error
    se = e.^2; %squared forecast error

    results.MFE{i} = mean(e); %mean forecast error
    results.MAFE{i} = mean(ae); %mean absolute FE
    results.MSFE{i} = mean(se); %mean squared FE
    results.RMSFE{i} = sqrt(results.MSFE{i}); %root mean squared FE
    results.medianSFE{i} = median(se); %median squared FE
end

%{
r = triu(qr(x,0));
xpxi = (r'*r)\eye(nvar);

results.beta    = xpxi*(x'*y);
results.yhat    = x*results.beta;
results.resid   = y - results.yhat;
sigu = results.resid'*results.resid;
results.sige    = sigu/(nobs-nvar);

% perform White's correction
xuux = mcov(x,results.resid);
xpxia = xpxi*xuux*xpxi;
results.covar = xpxia; %heteroskedasticity-robust variance-covariance 
tmp = sqrt(diag(xpxia));
results.bstd  = tmp; %heteroskedasticity-robust std dev for bhat
results.tstat = results.beta./tmp;
ym = y - ones(nobs,1)*mean(y);
rsqr1 = sigu;
rsqr2 = ym'*ym;
results.rsqr = 1.0 - rsqr1/rsqr2; % r-squared
rsqr1 = rsqr1/(nobs-nvar);
rsqr2 = rsqr2/(nobs-1.0);
results.rbar = 1 - (rsqr1/rsqr2); % rbar-squared
ediff = results.resid(2:nobs) - results.resid(1:nobs-1);
results.dw = diag((ediff'*ediff)./(sigu))'; % durbin-watson



function xuux = mcov(x,u)
% PURPOSE: computes x'*u*u'*x 
%----------------------------------------------------------------
% USAGE:   xuux = mcov(x,u);
% where: x = nobs x nvar explanatory variables matrix
%        u = nobs x 1 residuals
%----------------------------------------------------------------
% RETURNS: xuux such that xpx-inverse*xuux*xpx-inverse
%          represents a heteroscedasticity consistent vcv matrix
%----------------------------------------------------------------
% References: H. White 1980, Econometrica Vol. 48 pp. 818-838. 
%----------------------------------------------------------------

% written by:
% James P. LeSage, Dept of Economics
% University of Toledo
% 2801 W. Bancroft St,
% Toledo, OH 43606
% jlesage@spatial-econometrics.com

if nargin ~= 2
error('Wrong # of arguments to mcov');
end;

[nobs nvar] = size(x);

xuux = zeros(nvar,nvar);

for i=1:nobs;
xp = x(i,:);
xpx = xp'*xp;
upu = u(i,1)*u(i,1);
xuux = xuux + upu*xpx;
end;
%}
