function results=tslsNW4(y,y1,x1,x2,T)
% PURPOSE: computes Two-Stage Least-squares Regression With NW SE
%---------------------------------------------------
% USAGE: results = tsls(y,yendog,xexog,z)
% where: y      = dependent variable vector (nobs x 1)
%        y1 = endogenous variables matrix (nobs x g)
%        x1  = exogenous variables matrix for this equation (including ones
%        for the intercept)
%        x2   = all instruments 
%        T = table of data
%        nlag = lag length to use
%      t-stat: Fed = 1, alpha = 0, private = 0           
%---------------------------------------------------
% RETURNS: a structure
%        results.meth  = 'tsls'
%        results.bhat  = bhat estimates
%        results.tstat = t-statistics
%        results.yhat  = yhat predicted values
%        results.resid = residuals
%        results.sige  = e'*e/(n-k)
%        results.rsqr  = rsquared
%        results.rbar  = rbar-squared
%        results.dw    = Durbin-Watson Statistic
%        results.nobs  = nobs,
%        results.nendog = # of endogenous
%        results.nexog  = # of exogenous
%        results.nvar   = results.nendog + results.nexog
%        results.y      = y data vector
% --------------------------------------------------
% NOTE: you need to put a constant term in the x1 and z matrices
% --------------------------------------------------
% SEE ALSO: prt_reg(results), plt_reg(results)
%---------------------------------------------------

% written by:
% James P. LeSage, Dept of Economics
% University of Toledo
% jlesage@spatial-econometrics.com
% Edited by: Amy Guisinger - 9/25/2020

% Edited from Spatial Econometrics Toolbox by James P. LeSage  

T_new = rmmissing(T);

% Remove missing values, if any
wasnan = (isnan(y) | any(isnan(x2),2) | any(isnan(y1),2) | any(isnan(x1),2));
havenans = any(wasnan);
if havenans
   y(wasnan) = [];
   x2(wasnan,:) = [];
   y1(wasnan,:) = [];
   x1(wasnan,:) = [];
   n = length(y);
end

if (nargin ~= 5); error('Wrong # of arguments to tsls'); end;
results.meth = 'tsls';
[nobs1 g] = size(y1);
[nobs2 k] = size(x1);
[nobs3 l] = size(x2);
results.nendog = g; results.nexog = k; results.nvar = k+g;
if nobs1 == nobs2;
 if nobs2 == nobs3
 nobs = nobs1;
 end;
else
error('tsls: # of observations in yendog, xexog, z not the same');
end;
results.y = y; results.nobs = nobs;
% z contains all explanatory variables
% x1 contains exogenous
% y1 contains endogenous
% x2 contains instruments
x = [y1 x1];
z = [x1 x2];

xapxa = inv(z'*z);
I = eye(nobs);
MZ = I - z*xapxa*z';

%if 2sls
kappa = 1;

%if liml
%x1px1 = inv(x1'*x1);
%MX1 = I - x1*x1px1*x1';
%more

mid = I - kappa*MZ;
xpx = x'*mid*x;
xpxi = inv(xpx);
xpy = x'*mid*y;
% form xpx
%xpx = [y1'*z*xapxa*z'*y1     y1'*x1
%       x1'*y1                      x1'*x1];
%xpy = [y1'*z*xapxa*z'*y
%       x1'*y                  ];
%xpxi = inv(xpx);  

results.xpx = xpx;
results.xpy = xpy;
results.xpxi = xpxi;

results.beta  = xpxi*xpy;             % bhat
%results.yhat  = [y1 x1]*results.beta; % yhat
results.yhat  = x*results.beta; % yhat
results.resid = y - results.yhat;     % residuals
sigu = results.resid'*results.resid;
results.sige = sigu/(nobs-k-g);       % sige


%Regression Results
ypy = y'*y;
onesy = (ones(nobs,1)'*y)^2;
TSS = ypy-(onesy/nobs); % total sum of squares
results.TSS = TSS;
bxy = 2*results.beta'*x'*y;
bxxb = results.beta'*x'*x*results.beta;
ESS = ypy - bxy + bxxb; % error sum of squares
results.ESS = ESS;
MSS = TSS - ESS; % model sum of squares
results.MSS = MSS;
MSE = ESS/(nobs - k - g); % mean squared error
results.MSE = MSE;
rsqr = 1 - (ESS/TSS); % R-squared
results.rsqr = rsqr;
rsqra = 1 - (1 - rsqr)*(nobs-1)/(nobs - k - g); %adjusted R-squared
results.rbar = rsqra;
ediff = results.resid(2:nobs) - results.resid(1:nobs-1);
results.dw = (ediff'*ediff)/sigu; % durbin-watson

p =  x*xpxi*x'*z*xapxa*z'; %Projection matrix
results.lev   = diag(p); %leverage

%pa = x'*z*xapxa*z'*x;
%pai = inv(pa);
%p2 =  x*pai*x'*z*xapxa*z'; %Projection matrix
%results.lev2   = diag(p2); %leverage

nvar = k+g;
se_e = sqrt(results.sige*(1-results.lev));
%results.se_e = se_e;
stdres = results.resid ./ se_e;
%results.stdres = stdres;
stdres2 = stdres .* stdres;
h1h = results.lev ./ (1-results.lev);
d = (1/(nvar))*stdres2 .* h1h;
results.d = d; %Cook's distance

T_new = addvars(T_new,results.lev,results.d);
T_new.Properties.VariableNames{'Var8'} = 'leverage';
T_new.Properties.VariableNames{'Var9'} = 'Cookd';

results.T       = T_new;

A = size(d,1);
avg_lev = (1/nobs)*sum(results.lev);
results.avg_lev = avg_lev;
a = 2*avg_lev;
high_lev = zeros(A,1);
for i = 1:nobs
    if results.lev(i,1) > a
        high_lev(i,1) = 1;
    end
end
high_lev_sum = sum(high_lev); % total high leverage

d_mod = zeros(A,1);
for i = 1:nobs
    if d(i,1) > 0.5
    d_mod(i,1) = 1;
    end
end    
d_mod_sum = sum(d_mod); %total flagged moderate


d_extreme = zeros(A,1);
for i = 1:nobs
    if d(i,1) > 1
        d_extreme(i,1) = 1;
    end
end   
d_extreme_sum = sum(d_extreme); %total flagged extreme

results.high_lev =high_lev;
results.d_mod = d_mod;
results.d_extreme = d_extreme;
results.high_lev_sum = high_lev_sum;
results.d_mod_sum = d_mod_sum;
results.d_extreme_sum = d_extreme_sum;

%{
%homoskedastic IV SE
%varb = MSE*xpxi; % variance of b
varb = MSE*diag(xpxi); % variance of b
%results.bstd = sqrt(diag(varb));
results.bstd = sqrt(varb);
results.tstat = results.beta./(results.bstd);
%}

%{
% perform Robust SE

xz = x'*z;
P = xz*xapxa;
xhatp = P*z';
xhat = xhatp';
results.xhat = xhat;

q = nobs/(nobs-k-g);    %a constant finite-sample adjustment
%u = results.resid*xhat;
A = P*xz';
V = inv(A);

xuux = mcov(xhat,results.resid);
xpxia = V*xuux*V; 
%xpxia = q*V*xuux*V;              %need sample adjustment (i.e., q)?
%xpxia = xpxi*xuux*xpxi;
results.covar = xpxia; %heteroskedasticity-robust variance-covariance 
tmp = sqrt(diag(xpxia));
results.bstd  = tmp; %heteroskedasticity-robust std dev for bhat
results.tstat = results.beta./tmp;

function xuux = mcov(x,u)
% PURPOSE: computes x'*u*u'*x 
%----------------------------------------------------------------
% USAGE:   xuux = mcov(x,u);
% where: x = nobs x nvar explanatory variables matrix
%        u = nobs x 1 residuals
%----------------------------------------------------------------
% RETURNS: xuux such that xpx-inverse*xuux*xpx-inverse
%          represents a heteroscedasticity consistent vcv matrix
%----------------------------------------------------------------
% References: H. White 1980, Econometrica Vol. 48 pp. 818-838. 
%----------------------------------------------------------------

% written by:
% James P. LeSage, Dept of Economics
% University of Toledo
% 2801 W. Bancroft St,
% Toledo, OH 43606
% jlesage@spatial-econometrics.com

if nargin ~= 2
error('Wrong # of arguments to mcov');
end;

[nobs nvar] = size(x);

xuux = zeros(nvar,nvar);

for i=1:nobs;
xp = x(i,:);
xpx = xp'*xp;
upu = u(i,1)*u(i,1);
xuux = xuux + upu*xpx;
end;
%}

%%% NW 1994 suggests a bandwidth of {4(N/100)^(2/9)} + 1
nwaa = (nobs/100)^(2/9);
nwnlagaa = (4*nwaa) + 1;
nwnlag = round(nwnlagaa);

% perform Newey-West correction
nvar = k+g;
xz = x'*z;
P = xz*xapxa;
xhatp = P*z';
xhat = xhatp';
results.xhat = xhat;
A = P*xz';
invA = inv(A);

emat = [];
for i=1:nvar;
emat = [emat
        results.resid'];
end;
       
    hhat=emat.*xhat';
    G=zeros(nvar,nvar); w=zeros(2*nwnlag+1,1);
    a=0;

    while a~=nwnlag+1;
        ga=zeros(nvar,nvar);
        w(nwnlag+1+a,1)=(nwnlag+1-a)/(nwnlag+1);
        za=hhat(:,(a+1):nobs)*hhat(:,1:nobs-a)';
          if a==0;
           ga=ga+za;
          else
           ga=ga+za+za';
          end;
        G=G+w(nwnlag+1+a,1)*ga;
        a=a+1;
    end; % end of while
    
        V=xpxi*G*xpxi;
        %V=invA*G*invA; %same as above
        nwerr= sqrt(diag(V));
        
%results.tstat = results.beta./nwerr; % Newey-West t-statistics
results.tstat(1,1) = (results.beta(1,1) - 1)./nwerr(1,1); % Newey-West t-statistics Fed = 1
results.tstat(2:3,1) = results.beta(2:3,1)./nwerr(2:3,1); % Newey-West t-statistics
results.pvalue = tdis_prb(results.tstat,nobs-nvar); % find t-stat probabilities
results.nwerr = nwerr; % NW standard error
results.bstd = nwerr;
results.covar = V; %variance covariance matrix
%{
ym = y - ones(nobs,1)*mean(y);
rsqr1 = sigu;
rsqr2 = ym'*ym;
results.rsqr = 1.0 - rsqr1/rsqr2; % r-squared
rsqr1 = rsqr1/(nobs-nvar);
rsqr2 = rsqr2/(nobs-1.0);
results.rbar = 1 - (rsqr1/rsqr2); % rbar-squared
ediff = results.resid(2:nobs) - results.resid(1:nobs-1);
results.dw = diag((ediff'*ediff)./(sigu))'; % durbin-watson
%}
